@@ -1,3 +1,5 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
+
 /* Metacity interface used by GTK+ UI to talk to core */
 
 /* 
@@ -27,6 +29,17 @@
 #include "workspace.h"
 #include "prefs.h"
 
+/* Looks up the MetaWindow representing the frame of the given X window.
+ * Used as a helper function by a bunch of the functions below.
+ *
+ * FIXME: The functions that use this function throw the result away
+ * after use. Many of these functions tend to be called in small groups,
+ * which results in get_window() getting called several times in succession
+ * with the same parameters. We should profile to see whether this wastes
+ * much time, and if it does we should look into a generalised
+ * meta_core_get_window_info() which takes a bunch of pointers to variables
+ * to put its results in, and only fills in the non-null ones.
+ */
 static MetaWindow *
 get_window (Display *xdisplay,
 	    Window   frame_xwindow)
@@ -60,6 +73,19 @@ meta_core_get_client_size (Display *xdisplay,
     *height = window->rect.height;
 }
 
+gboolean
+meta_core_window_has_frame (Display *xdisplay,
+                            Window frame_xwindow)
+{
+  MetaDisplay *display;
+  MetaWindow *window;
+  
+  display = meta_display_for_x_display (xdisplay);
+  window = meta_display_lookup_x_window (display, frame_xwindow);
+
+  return window != NULL && window->frame != NULL;
+}
+
 gboolean
 meta_core_titlebar_is_onscreen (Display *xdisplay,
                                 Window   frame_xwindow)
@@ -199,7 +225,7 @@ meta_core_user_raise (Display *xdisplay,
 void
 meta_core_user_lower_and_unfocus (Display *xdisplay,
                                   Window   frame_xwindow,
-                                  Time     timestamp)
+                                  guint32  timestamp)
 {
   MetaWindow *window = get_window (xdisplay, frame_xwindow);
   
@@ -242,7 +268,7 @@ meta_core_user_lower_and_unfocus (Display *xdisplay,
 void
 meta_core_user_focus (Display *xdisplay,
                       Window   frame_xwindow,
-                      Time     timestamp)
+                      guint32  timestamp)
 {
   MetaWindow *window = get_window (xdisplay, frame_xwindow);
   
@@ -339,20 +365,22 @@ meta_core_delete (Display *xdisplay,
 
 void
 meta_core_unshade (Display *xdisplay,
-                   Window   frame_xwindow)
+                   Window   frame_xwindow,
+                   guint32  timestamp)
 {
   MetaWindow *window = get_window (xdisplay, frame_xwindow);
 
-  meta_window_unshade (window);
+  meta_window_unshade (window, timestamp);
 }
 
 void
 meta_core_shade (Display *xdisplay,
-                 Window   frame_xwindow)
+                 Window   frame_xwindow,
+                 guint32  timestamp)
 {
   MetaWindow *window = get_window (xdisplay, frame_xwindow);
   
-  meta_window_shade (window);
+  meta_window_shade (window, timestamp);
 }
 
 void
@@ -364,6 +392,24 @@ meta_core_unstick (Display *xdisplay,
   meta_window_unstick (window);
 }
 
+void
+meta_core_make_above (Display *xdisplay,
+                      Window   frame_xwindow)
+{
+  MetaWindow *window = get_window (xdisplay, frame_xwindow);
+
+  meta_window_make_above (window);
+}
+
+void
+meta_core_unmake_above (Display *xdisplay,
+                        Window   frame_xwindow)
+{
+  MetaWindow *window = get_window (xdisplay, frame_xwindow);
+
+  meta_window_unmake_above (window);
+}
+
 void
 meta_core_stick (Display *xdisplay,
                  Window   frame_xwindow)
@@ -441,7 +487,7 @@ meta_core_show_window_menu (Display *xdisplay,
                             int      root_x,
                             int      root_y,
                             int      button,
-                            Time     timestamp)
+                            guint32  timestamp)
 {
   MetaWindow *window = get_window (xdisplay, frame_xwindow);
   
@@ -584,10 +630,11 @@ meta_core_begin_grab_op (Display    *xdisplay,
                          Window      frame_xwindow,
                          MetaGrabOp  op,
                          gboolean    pointer_already_grabbed,
+                         gboolean    frame_action,
                          int         event_serial,
                          int         button,
                          gulong      modmask,
-                         Time        timestamp,
+                         guint32     timestamp,
                          int         root_x,
                          int         root_y)
 {
@@ -602,6 +649,7 @@ meta_core_begin_grab_op (Display    *xdisplay,
   
   return meta_display_begin_grab_op (display, screen, window,
                                      op, pointer_already_grabbed,
+                                     frame_action,
                                      event_serial,
                                      button, modmask,
                                      timestamp, root_x, root_y);
@@ -609,7 +657,7 @@ meta_core_begin_grab_op (Display    *xdisplay,
 
 void
 meta_core_end_grab_op (Display *xdisplay,
-                       Time     timestamp)
+                       guint32  timestamp)
 {
   MetaDisplay *display;
   
@@ -708,3 +756,31 @@ meta_core_increment_event_serial (Display *xdisplay)
   meta_display_increment_event_serial (display);
 }
 
+void
+meta_invalidate_default_icons (void)
+{
+  GSList *displays, *windows;
+
+  for (displays = meta_displays_list ();
+       displays != NULL;
+       displays = displays->next)
+    {
+
+      for (windows = meta_display_list_windows (displays->data);
+           windows != NULL;
+           windows = windows->next)
+        {
+
+          MetaWindow *window = (MetaWindow*)windows->data;
+
+          if (window->icon_cache.origin == USING_FALLBACK_ICON)
+            {
+              meta_icon_cache_free (&(window->icon_cache));
+              meta_window_update_icon_now (window);
+            }
+        }
+
+      g_slist_free (windows);
+    }
+}
+
