@@ -1,8 +1,11 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
+
 /* Metacity window menu */
 
 /* 
  * Copyright (C) 2001 Havoc Pennington
  * Copyright (C) 2004 Rob Adams
+ * Copyright (C) 2005 Elijah Newren
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -51,23 +54,41 @@ struct _MenuData
 static void activate_cb (GtkWidget *menuitem, gpointer data);
 
 static MenuItem menuitems[] = {
+  /* Translators: Translate this string the same way as you do in libwnck! */
   { META_MENU_OP_MINIMIZE, METACITY_STOCK_MINIMIZE, FALSE, N_("Mi_nimize") },
+  /* Translators: Translate this string the same way as you do in libwnck! */
   { META_MENU_OP_MAXIMIZE, METACITY_STOCK_MAXIMIZE, FALSE, N_("Ma_ximize") },
+  /* Translators: Translate this string the same way as you do in libwnck! */
   { META_MENU_OP_UNMAXIMIZE, NULL, FALSE, N_("Unma_ximize") },
+  /* Translators: Translate this string the same way as you do in libwnck! */
   { META_MENU_OP_SHADE, NULL, FALSE, N_("Roll _Up") },
+  /* Translators: Translate this string the same way as you do in libwnck! */
   { META_MENU_OP_UNSHADE, NULL, FALSE, N_("_Unroll") },
+  /* Translators: Translate this string the same way as you do in libwnck! */
   { META_MENU_OP_ABOVE, NULL, FALSE, N_("On _Top") },
+  /* Translators: Translate this string the same way as you do in libwnck! */
   { META_MENU_OP_UNABOVE, NULL, TRUE, N_("On _Top") },
+  /* Translators: Translate this string the same way as you do in libwnck! */
   { META_MENU_OP_MOVE, NULL, FALSE, N_("_Move") },
+  /* Translators: Translate this string the same way as you do in libwnck! */
   { META_MENU_OP_RESIZE, NULL, FALSE, N_("_Resize") },
+  /* Translators: Translate this string the same way as you do in libwnck! */
+  { META_MENU_OP_RECOVER, NULL, FALSE, N_("Move Titlebar On_screen") },
   { 0, NULL, FALSE, NULL }, /* separator */
+  /* Translators: Translate this string the same way as you do in libwnck! */
   { META_MENU_OP_DELETE, METACITY_STOCK_DELETE, FALSE, N_("_Close") },
   { META_MENU_OP_WORKSPACES, NULL, FALSE, NULL }, /* separator */
+  /* Translators: Translate this string the same way as you do in libwnck! */
   { META_MENU_OP_STICK, NULL, FALSE, N_("_Always on Visible Workspace") },
+  /* Translators: Translate this string the same way as you do in libwnck! */
   { META_MENU_OP_UNSTICK, NULL, FALSE,  N_("_Only on This Workspace") },
+  /* Translators: Translate this string the same way as you do in libwnck! */
   { META_MENU_OP_MOVE_LEFT, NULL, FALSE, N_("Move to Workspace _Left") },
+  /* Translators: Translate this string the same way as you do in libwnck! */
   { META_MENU_OP_MOVE_RIGHT, NULL, FALSE, N_("Move to Workspace R_ight") },
+  /* Translators: Translate this string the same way as you do in libwnck! */
   { META_MENU_OP_MOVE_UP, NULL, FALSE, N_("Move to Workspace _Up") },
+  /* Translators: Translate this string the same way as you do in libwnck! */
   { META_MENU_OP_MOVE_DOWN, NULL, FALSE, N_("Move to Workspace _Down") }
 };
 
@@ -265,6 +286,9 @@ meta_window_menu_new   (MetaFrames         *frames,
 
   if (n_workspaces < 2)
     ops &= ~(META_MENU_OP_STICK | META_MENU_OP_UNSTICK | META_MENU_OP_WORKSPACES);
+  else if (n_workspaces == 2) 
+    /* #151183: If we only have two workspaces, disable the menu listing them. */
+    ops &= ~(META_MENU_OP_WORKSPACES);
   
   menu = g_new (MetaWindowMenu, 1);
   menu->frames = frames;
