@@ -1,3 +1,5 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
+
 /* Metacity preferences */
 
 /* 
@@ -1379,10 +1381,42 @@ button_function_from_string (const char *str)
     return META_BUTTON_FUNCTION_MAXIMIZE;
   else if (strcmp (str, "close") == 0)
     return META_BUTTON_FUNCTION_CLOSE;
-  else
+  else if (strcmp (str, "shade") == 0)
+    return META_BUTTON_FUNCTION_SHADE;
+  else if (strcmp (str, "above") == 0)
+    return META_BUTTON_FUNCTION_ABOVE;
+  else if (strcmp (str, "stick") == 0)
+    return META_BUTTON_FUNCTION_STICK;
+  else 
+    /* don't know; give up */
     return META_BUTTON_FUNCTION_LAST;
 }
 
+static MetaButtonFunction
+button_opposite_function (MetaButtonFunction ofwhat)
+{
+  switch (ofwhat)
+    {
+    case META_BUTTON_FUNCTION_SHADE:
+      return META_BUTTON_FUNCTION_UNSHADE;
+    case META_BUTTON_FUNCTION_UNSHADE:
+      return META_BUTTON_FUNCTION_SHADE;
+
+    case META_BUTTON_FUNCTION_ABOVE:
+      return META_BUTTON_FUNCTION_UNABOVE;
+    case META_BUTTON_FUNCTION_UNABOVE:
+      return META_BUTTON_FUNCTION_ABOVE;
+
+    case META_BUTTON_FUNCTION_STICK:
+      return META_BUTTON_FUNCTION_UNSTICK;
+    case META_BUTTON_FUNCTION_UNSTICK:
+      return META_BUTTON_FUNCTION_STICK;
+
+    default:
+      return META_BUTTON_FUNCTION_LAST;
+    }
+}
+
 static gboolean
 update_button_layout (const char *value)
 {
@@ -1433,6 +1467,11 @@ update_button_layout (const char *value)
               new_layout.left_buttons[i] = f;
               used[f] = TRUE;
               ++i;
+
+              f = button_opposite_function (f);
+
+              if (f != META_BUTTON_FUNCTION_LAST)
+                new_layout.left_buttons[i++] = f;
             }
           else
             {
@@ -1471,7 +1510,12 @@ update_button_layout (const char *value)
               new_layout.right_buttons[i] = f;
               used[f] = TRUE;
               ++i;
-            }
+
+              f = button_opposite_function (f);
+
+              if (f != META_BUTTON_FUNCTION_LAST)
+                new_layout.right_buttons[i++] = f;
+             }
           else
             {
               meta_topic (META_DEBUG_PREFS, "Ignoring unknown or already-used button name \"%s\"\n",
@@ -2151,6 +2195,7 @@ update_binding (MetaKeyPref *binding,
                 const char  *value)
 {
   unsigned int keysym;
+  unsigned int keycode;
   MetaVirtualModifier mods;
   MetaKeyCombo *combo;
   gboolean changed;
@@ -2160,10 +2205,11 @@ update_binding (MetaKeyPref *binding,
               binding->name, value ? value : "none");
   
   keysym = 0;
+  keycode = 0;
   mods = 0;
   if (value)
     {
-      if (!meta_ui_parse_accelerator (value, &keysym, &mods))
+      if (!meta_ui_parse_accelerator (value, &keysym, &keycode, &mods))
         {
           meta_topic (META_DEBUG_KEYBINDINGS,
                       "Failed to parse new gconf value\n");
@@ -2246,16 +2292,19 @@ update_binding (MetaKeyPref *binding,
   
   changed = FALSE;
   if (keysym != combo->keysym ||
+      keycode != combo->keycode ||
       mods != combo->modifiers)
     {
       changed = TRUE;
       
       combo->keysym = keysym;
+      combo->keycode = keycode;
       combo->modifiers = mods;
       
       meta_topic (META_DEBUG_KEYBINDINGS,
-                  "New keybinding for \"%s\" is keysym = 0x%x mods = 0x%x\n",
-                  binding->name, combo->keysym, combo->modifiers);
+                  "New keybinding for \"%s\" is keysym = 0x%x keycode = 0x%x mods = 0x%x\n",
+                  binding->name, combo->keysym, combo->keycode,
+                  combo->modifiers);
     }
   else
     {
@@ -2272,6 +2321,7 @@ update_list_binding (MetaKeyPref *binding,
                      MetaStringListType type_of_value)
 {
   unsigned int keysym;
+  unsigned int keycode;
   MetaVirtualModifier mods;
   gboolean changed = FALSE;
   const gchar *pref_string;
@@ -2308,6 +2358,7 @@ update_list_binding (MetaKeyPref *binding,
   while (pref_iterator)
     {
       keysym = 0;
+      keycode = 0;
       mods = 0;
 
       if (!pref_iterator->data)
@@ -2328,7 +2379,7 @@ update_list_binding (MetaKeyPref *binding,
           g_assert_not_reached ();
         }
       
-      if (!meta_ui_parse_accelerator (pref_string, &keysym, &mods))
+      if (!meta_ui_parse_accelerator (pref_string, &keysym, &keycode, &mods))
         {
           meta_topic (META_DEBUG_KEYBINDINGS,
                       "Failed to parse new gconf value\n");
@@ -2363,12 +2414,13 @@ update_list_binding (MetaKeyPref *binding,
 
       combo = g_malloc0 (sizeof (MetaKeyCombo));
       combo->keysym = keysym;
+      combo->keycode = keycode;
       combo->modifiers = mods;
       binding->bindings->next = g_slist_prepend (binding->bindings->next, combo);
 
       meta_topic (META_DEBUG_KEYBINDINGS,
-                      "New keybinding for \"%s\" is keysym = 0x%x mods = 0x%x\n",
-                      binding->name, keysym, mods);
+                      "New keybinding for \"%s\" is keysym = 0x%x keycode = 0x%x mods = 0x%x\n",
+                      binding->name, keysym, keycode, mods);
 
       pref_iterator = pref_iterator->next;
     }  
