@@ -1,3 +1,5 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
+
 /* Metacity interface for talking to GTK+ UI module */
 
 /* 
@@ -28,16 +30,20 @@
 #include "menu.h"
 #include "core.h"
 #include "theme.h"
-
-#include "eggaccelerators.h"
+#include "iconcache.h"
 
 #include "inlinepixbufs.h"
 
 #include <pango/pangox.h>
 
 #include <string.h>
+#include <stdlib.h>
 
 static void meta_stock_icons_init (void);
+static void meta_ui_accelerator_parse (const char      *accel,
+                                       guint           *keysym,
+                                       guint           *keycode,
+                                       GdkModifierType *keymask);
 
 struct _MetaUI
 {
@@ -56,12 +62,9 @@ meta_ui_init (int *argc, char ***argv)
 }
 
 Display*
-meta_ui_get_display (const char *name)
+meta_ui_get_display (void)
 {
-  if (name == NULL)
-    return gdk_display;
-  else
-    return NULL;
+  return gdk_display;
 }
 
 typedef struct _EventFunc EventFunc;
@@ -73,26 +76,29 @@ struct _EventFunc
   int last_event_serial;
 };
 
+static EventFunc *ef = NULL;
+
 static GdkFilterReturn
 filter_func (GdkXEvent *xevent,
              GdkEvent *event,
              gpointer data)
 {
-  EventFunc *ef;
-
-  ef = data;
+  g_return_val_if_fail (ef != NULL, GDK_FILTER_CONTINUE);
 
   if ((* ef->func) (xevent, ef->data))
     return GDK_FILTER_REMOVE;
   else
     {
-      ef->last_event_serial = ((XEvent*)xevent)->xany.serial;
+      /* ef would be NULL here if we removed the filter function
+       * in response to the event.
+       */
+      if (ef != NULL)
+        ef->last_event_serial = ((XEvent*)xevent)->xany.serial;
+
       return GDK_FILTER_CONTINUE;
     }
 }
 
-static EventFunc *ef = NULL;
-
 void
 meta_ui_add_event_func (Display       *xdisplay,
                         MetaEventFunc  func,
@@ -731,6 +737,7 @@ meta_ui_set_current_theme (const char *name,
                            gboolean    force_reload)
 {
   meta_theme_set_current (name, force_reload);
+  meta_invalidate_default_icons ();
 }
 
 gboolean
@@ -739,50 +746,71 @@ meta_ui_have_a_theme (void)
   return meta_theme_get_current () != NULL;
 }
 
+static void
+meta_ui_accelerator_parse (const char      *accel,
+                           guint           *keysym,
+                           guint           *keycode,
+                           GdkModifierType *keymask)
+{
+  if (accel[0] == '0' && accel[1] == 'x')
+    {
+      *keysym = 0;
+      *keycode = (guint) strtoul (accel, NULL, 16);
+      *keymask = 0;
+    }
+  else
+    gtk_accelerator_parse (accel, keysym, keymask);
+}
+
 gboolean
 meta_ui_parse_accelerator (const char          *accel,
                            unsigned int        *keysym,
+                           unsigned int        *keycode,
                            MetaVirtualModifier *mask)
 {
-  EggVirtualModifierType gdk_mask = 0;
+  GdkModifierType gdk_mask = 0;
   guint gdk_sym = 0;
+  guint gdk_code = 0;
   
   *keysym = 0;
+  *keycode = 0;
   *mask = 0;
 
   if (strcmp (accel, "disabled") == 0)
     return TRUE;
   
-  if (!egg_accelerator_parse_virtual (accel, &gdk_sym, &gdk_mask))
+  meta_ui_accelerator_parse (accel, &gdk_sym, &gdk_code, &gdk_mask);
+  if (gdk_mask == 0 && gdk_sym == 0 && gdk_code == 0)
     return FALSE;
 
-  if (gdk_sym == None)
+  if (gdk_sym == None && gdk_code == 0)
     return FALSE;
   
-  if (gdk_mask & EGG_VIRTUAL_RELEASE_MASK) /* we don't allow this */
+  if (gdk_mask & GDK_RELEASE_MASK) /* we don't allow this */
     return FALSE;
   
   *keysym = gdk_sym;
+  *keycode = gdk_code;
 
-  if (gdk_mask & EGG_VIRTUAL_SHIFT_MASK)
+  if (gdk_mask & GDK_SHIFT_MASK)
     *mask |= META_VIRTUAL_SHIFT_MASK;
-  if (gdk_mask & EGG_VIRTUAL_CONTROL_MASK)
+  if (gdk_mask & GDK_CONTROL_MASK)
     *mask |= META_VIRTUAL_CONTROL_MASK;
-  if (gdk_mask & EGG_VIRTUAL_ALT_MASK)
+  if (gdk_mask & GDK_MOD1_MASK)
     *mask |= META_VIRTUAL_ALT_MASK;
-  if (gdk_mask & EGG_VIRTUAL_MOD2_MASK)
+  if (gdk_mask & GDK_MOD2_MASK)
     *mask |= META_VIRTUAL_MOD2_MASK;
-  if (gdk_mask & EGG_VIRTUAL_MOD3_MASK)
+  if (gdk_mask & GDK_MOD3_MASK)
     *mask |= META_VIRTUAL_MOD3_MASK;
-  if (gdk_mask & EGG_VIRTUAL_MOD4_MASK)
+  if (gdk_mask & GDK_MOD4_MASK)
     *mask |= META_VIRTUAL_MOD4_MASK;
-  if (gdk_mask & EGG_VIRTUAL_MOD5_MASK)
+  if (gdk_mask & GDK_MOD5_MASK)
     *mask |= META_VIRTUAL_MOD5_MASK;
-  if (gdk_mask & EGG_VIRTUAL_SUPER_MASK)
+  if (gdk_mask & GDK_SUPER_MASK)
     *mask |= META_VIRTUAL_SUPER_MASK;
-  if (gdk_mask & EGG_VIRTUAL_HYPER_MASK)
+  if (gdk_mask & GDK_HYPER_MASK)
     *mask |= META_VIRTUAL_HYPER_MASK;
-  if (gdk_mask & EGG_VIRTUAL_META_MASK)
+  if (gdk_mask & GDK_META_MASK)
     *mask |= META_VIRTUAL_META_MASK;
   
   return TRUE;
@@ -793,7 +821,7 @@ gchar*
 meta_ui_accelerator_name  (unsigned int        keysym,
                            MetaVirtualModifier mask)
 {
-  EggVirtualModifierType mods = 0;
+  GdkModifierType mods = 0;
         
   if (keysym == 0 && mask == 0)
     {
@@ -801,27 +829,27 @@ meta_ui_accelerator_name  (unsigned int        keysym,
     }
 
   if (mask & META_VIRTUAL_SHIFT_MASK)
-    mods |= EGG_VIRTUAL_SHIFT_MASK;
+    mods |= GDK_SHIFT_MASK;
   if (mask & META_VIRTUAL_CONTROL_MASK)
-    mods |= EGG_VIRTUAL_CONTROL_MASK;
+    mods |= GDK_CONTROL_MASK;
   if (mask & META_VIRTUAL_ALT_MASK)
-    mods |= EGG_VIRTUAL_ALT_MASK;
+    mods |= GDK_MOD1_MASK;
   if (mask & META_VIRTUAL_MOD2_MASK)
-    mods |= EGG_VIRTUAL_MOD2_MASK;
+    mods |= GDK_MOD2_MASK;
   if (mask & META_VIRTUAL_MOD3_MASK)
-    mods |= EGG_VIRTUAL_MOD3_MASK;
+    mods |= GDK_MOD3_MASK;
   if (mask & META_VIRTUAL_MOD4_MASK)
-    mods |= EGG_VIRTUAL_MOD4_MASK;
+    mods |= GDK_MOD4_MASK;
   if (mask & META_VIRTUAL_MOD5_MASK)
-    mods |= EGG_VIRTUAL_MOD5_MASK;
+    mods |= GDK_MOD5_MASK;
   if (mask & META_VIRTUAL_SUPER_MASK)
-    mods |= EGG_VIRTUAL_SUPER_MASK;
+    mods |= GDK_SUPER_MASK;
   if (mask & META_VIRTUAL_HYPER_MASK)
-    mods |= EGG_VIRTUAL_HYPER_MASK;
+    mods |= GDK_HYPER_MASK;
   if (mask & META_VIRTUAL_META_MASK)
-    mods |= EGG_VIRTUAL_META_MASK;
+    mods |= GDK_META_MASK;
 
-  return egg_virtual_accelerator_name (keysym, mods);
+  return gtk_accelerator_name (keysym, mods);
 
 }
 
@@ -829,42 +857,44 @@ gboolean
 meta_ui_parse_modifier (const char          *accel,
                         MetaVirtualModifier *mask)
 {
-  EggVirtualModifierType gdk_mask = 0;
+  GdkModifierType gdk_mask = 0;
   guint gdk_sym = 0;
+  guint gdk_code = 0;
   
   *mask = 0;
 
   if (strcmp (accel, "disabled") == 0)
     return TRUE;
   
-  if (!egg_accelerator_parse_virtual (accel, &gdk_sym, &gdk_mask))
+  meta_ui_accelerator_parse (accel, &gdk_sym, &gdk_code, &gdk_mask);
+  if (gdk_mask == 0 && gdk_sym == 0 && gdk_code == 0)
     return FALSE;
 
-  if (gdk_sym != None)
+  if (gdk_sym != None || gdk_code != 0)
     return FALSE;
   
-  if (gdk_mask & EGG_VIRTUAL_RELEASE_MASK) /* we don't allow this */
+  if (gdk_mask & GDK_RELEASE_MASK) /* we don't allow this */
     return FALSE;
 
-  if (gdk_mask & EGG_VIRTUAL_SHIFT_MASK)
+  if (gdk_mask & GDK_SHIFT_MASK)
     *mask |= META_VIRTUAL_SHIFT_MASK;
-  if (gdk_mask & EGG_VIRTUAL_CONTROL_MASK)
+  if (gdk_mask & GDK_CONTROL_MASK)
     *mask |= META_VIRTUAL_CONTROL_MASK;
-  if (gdk_mask & EGG_VIRTUAL_ALT_MASK)
+  if (gdk_mask & GDK_MOD1_MASK)
     *mask |= META_VIRTUAL_ALT_MASK;
-  if (gdk_mask & EGG_VIRTUAL_MOD2_MASK)
+  if (gdk_mask & GDK_MOD2_MASK)
     *mask |= META_VIRTUAL_MOD2_MASK;
-  if (gdk_mask & EGG_VIRTUAL_MOD3_MASK)
+  if (gdk_mask & GDK_MOD3_MASK)
     *mask |= META_VIRTUAL_MOD3_MASK;
-  if (gdk_mask & EGG_VIRTUAL_MOD4_MASK)
+  if (gdk_mask & GDK_MOD4_MASK)
     *mask |= META_VIRTUAL_MOD4_MASK;
-  if (gdk_mask & EGG_VIRTUAL_MOD5_MASK)
+  if (gdk_mask & GDK_MOD5_MASK)
     *mask |= META_VIRTUAL_MOD5_MASK;
-  if (gdk_mask & EGG_VIRTUAL_SUPER_MASK)
+  if (gdk_mask & GDK_SUPER_MASK)
     *mask |= META_VIRTUAL_SUPER_MASK;
-  if (gdk_mask & EGG_VIRTUAL_HYPER_MASK)
+  if (gdk_mask & GDK_HYPER_MASK)
     *mask |= META_VIRTUAL_HYPER_MASK;
-  if (gdk_mask & EGG_VIRTUAL_META_MASK)
+  if (gdk_mask & GDK_META_MASK)
     *mask |= META_VIRTUAL_META_MASK;
   
   return TRUE;
